#include <raylib.h>
#include <stdio.h>

typedef struct ParserState
{
  char *input;
  int position;
  char nextToken[256];
} ParserState;

void ParserStateSkipWhiteSpaces(ParserState *state)
{
  char *input = state->input;
  int pos = state->position;
  int skipped = 1;
  while (skipped)
  {
    skipped = 0;
    if (input[pos] == '-' && input[pos + 1] == '-')
    {
      skipped = 1;
      // skip comments
      while (input[pos] != 0 && input[pos] != '\n')
      {
        pos++;
      }
    }
  
    // skip white spaces and ignore colons
    while (input[pos] != 0 && (input[pos] <= ' ' || input[pos] == ':'))
    {
      skipped = 1;
      pos++;
    }

    // repeat until no more white spaces or comments
  }
  state->position = pos;
}

int ParserStateReadNextToken(ParserState *state)
{
  ParserStateSkipWhiteSpaces(state);

  int i = 0, pos = state->position;
  char *input = state->input;

  // read token
  while (input[pos] != 0 && input[pos] > ' ' && input[pos] != ':' && i < 256)
  {
    state->nextToken[i] = input[pos];
    pos++;
    i++;
  }
  state->position = pos;

  if (i == 0 || i == 256)
  {
    state->nextToken[0] = 0;
    return 0;
  }
  // terminate the token
  state->nextToken[i] = 0;
  return 1;
}

int main()
{
  char *fileContent = LoadFileText("data/level.txt");
  if (fileContent == NULL)
  {
    printf("Failed to load file\n");
    return 1;
  }

  ParserState state = {fileContent, 0, {0}};
  for (int i = 0; i < 10; i++)
  {
    if (!ParserStateReadNextToken(&state))
    {
      break;
    }
    printf("Token: %s\n", state.nextToken);
  }

  printf("Parser reading stopped at position %d\n", state.position);
  
  UnloadFileText(fileContent);
  return 0; 
}
